@@ -4,7 +4,7 @@ use Exporter 'import';
 use Digest::MD5 qw(md5_hex);
 
 use 5.010;
-our $VERSION = '1.612'; # ==> ALSO update the version in the pod text below!
+our $VERSION = '1.613'; # ==> ALSO update the version in the pod text below!
 
 # loaded modules
 our %TAG_HANDLERS;
@@ -331,7 +331,7 @@ DTL::Fast - Perl implementation of Django templating language.
 
 =head1 VERSION
 
-Version 1.612
+Version 1.613
 
 =head1 SYNOPSIS
 
@@ -371,9 +371,9 @@ Goals of this implementation are:
 
 =head2 Current status
 
-Current release implements almost all tags and filters documented on Django site.
+Current release implements almost all tags and filters documented on Django site. Also, some extensions has been made to the tags, filters and operators.
 
-There are no significant speed optimizations done yet.
+Some optimisation has been done and some critical section been implemeted in C.
 
 Internationalization and localization are not yet implemented.
 
@@ -519,6 +519,25 @@ More about custom operators you may read in L</CUSTOM OPERATORS> section.
     
 Preloads all registered operators modules. Mostly for debugging purposes or persistent environment stability.
   
+=head2 html_protect
+
+    my $protected_text = DTL::Fast::html_protect($raw_text);
+
+This function protects string for a safe HTML output by replacing characters: E<lt> E<gt> & " ' with their HTML equivalents. Function written in C and works pretty fast. If you want it to properly treat UTF-8 characters you must set utf8 flag for C<$raw_text>.
+
+=head2 spaceless
+
+    my $spaceless_text = DTL::Fast::spaceless($raw_text);
+
+This function implements spaceless tag, by removing spaces, tabs and newlines between E<lt> and E<gt> symbols. Written in C and works pretty fast.If you want it to properly treat UTF-8 characters you must set utf8 flag for C<$raw_text>.
+
+=head2 eval_sequence
+
+    eval 'some code ... ';
+    my $sequence = DTL::Fast::eval_sequence();
+    
+This function returns internal perl eval counter. This counter being increased on every C<eval $string;> operation. Counter may be useful for debugging purposes with multiple evals, in order to detect in which exact eval error occured. Perl's C<die> method says smth like: C<error in eval(42) ...>. 42 in this example is eval sequence counter.
+  
 =head1 TEMPLATING LANGUAGE
 
 =head2 Tags
@@ -597,18 +616,211 @@ To do...
 
 =head1 CUSTOM TAGS
 
-To do...
+You may extend C<DTL::Fast> with your own custom tags. It's pretty simple process. There are two types of tags in the library: simple tags and block tags. Simple tag just doing something, like
+including file and block tags contains other blocks and affects their rendering in some way, like for loop.
+
+Every tag is a separate class inherited from certain prototype.
+
+=head2 Simple tags
+
+Every simple tag implemented as class, inherited from L<C<DTL::Fast::Tag::Simple>>:
+
+    package CustomTags::MyTag;                          # your tag class
+    use strict; use utf8; use warnings FATAL => 'all';  # make a clean code
+    use parent 'DTL::Fast::Tag::Simple';                # parent class
+    
+    $DTL::Fast::TAG_HANDLERS{'my_tag'} = __PACKAGE__;   # register your tag keyword with your package in tag handlers
+    
+    # You may need to override a constructor in case you need to support some complex syntax
+    # if you don't need to do something special, just don't override it
+    # invoked once on parse phase
+    sub new
+    {
+        my(
+            $proto,         # well, this is a class proto
+            $parameter,     # this is a text with everything after your tag: {% my_tag ...this is a parameter if any... %}
+            %kwargs         # additional named arguments may be passed to the constructor
+        ) = @_;
+        
+        ....
+        
+        my $self = $proto->SUPER::new($parameter, %kwargs);
+        
+        ....
+        
+        return $self;
+    }
+    
+    # Parameter parsing method, invoked once on parsing phase. Parent constructor stores passed $parameter
+    # into the $self->{'parameter'} hash entry and invokes parse_parameter method.
+    # here you may make some preparations for rendering, parse arguments and do whatever you want.
+    sub parse_parameters
+    {
+        my $self = shift;   # reference to the tag object
+        
+        ....
+        
+        return $self;       # IMPORTANT, method must return $self, because it's being used in chain call
+    }
+
+    # Rendering method. Invoked on every rendering iteration
+    # The main work is being done here
+    sub render
+    {
+        my(
+            $self,          # object reference
+            $context,       # context object
+            $global_safe    # this flag is set to true, if somehow global safety is turned on, means disabled HTML escaping
+        ) = @_;
+        
+        ...
+        
+        return $result;     # this is just a string generated by tag. If your tag is silent, just return an empty string
+    }
+    
+After creating your tag class, you need to register it in C<DTL::Fast>, so library knew which module to load on tag keyword:
+
+    use DTL::Fast qw(register_tag);
+
+    register_tag( 'my_tag' => 'CustomTags::MyTag' );
+    
+Basically, that's all. For more information you may view through the sources of buit-in simple tags.
+
+=head2 Block tags
+
+Every block tag in the library implemented as class, inherited from L<C<DTL::Fast::Tag>>:
+
+    package CustomTags::MyBlockTag;                     # your tag class
+    use strict; use utf8; use warnings FATAL => 'all';  # make a clean code
+    use parent 'DTL::Fast::Tag';                        # parent class is Tag, not Tag::Simple
+    
+    $DTL::Fast::TAG_HANDLERS{'my_block_tag'} = __PACKAGE__;   # register your tag opening keyword with your package in tag handlers
+
+    # constructor and parameters parser are the same as in simple tags    
+
+    # method must be defined and must return block close tag, so parser would know where block is ended    
+    sub get_close_tag{ return 'end_my_tag';}
+    
+    # optional. Method invoked on parsing phase with next parsed chunk of current block tag contents
+    # this method being used, for example, in the 'if' tag to push chunks into sub-branches, not if block itself
+    sub add_chunk
+    {
+        my(
+            $self,      # reference to the tag object
+            $chunk      # parsed chunk object, if chunk must be ignored, it would be undefined
+        ) = @_;
+        
+        ... here you may alter chunk or put it in some other place, like subblocks...
+        
+        return $self;   # don't forget to return self
+    }
+    
+    # optional. Method invoked on parsing phase for every {% ... .%} construction.
+    # if there is no special controlling keywords, don't override this method
+    sub parse_tag_chunk
+    {
+        my(
+            $self,      # reference to the tag object
+            $tag_name,  # tag keyword
+            $tag_param  # everything after tag keyword (constructor's $parameter)
+        ) = @_;
+        
+        my $result;
+        
+        # here you may interpret additional tag keywords, like else, elsif and so on.
+        if( $tag_name eq 'my_tag_else' )
+        {
+            ... do smth special 
+        }
+        elsif( $tag_name eq 'my_tag_alternative_end' )
+        {
+            $self->{'raw_chunks'} = []; # this construction ends current block parsing
+        }
+        else    # if it was not special keyword, just do regular work
+        {
+            $result = $self->SUPER::parse_tag_chunk($tag_name, $tag_param);
+        }
+        
+        return $result;  # result must be a generated chunk or undef if there is no one
+    }
+
+    # Rendering method. Invoked on every rendering iteration
+    # The main work is being done here
+    sub render
+    {
+        my(
+            $self,          # object reference
+            $context,       # context object
+            $global_safe    # this flag is set to true, if somehow global safety is turned on, means disabled HTML escaping
+        ) = @_;
+        
+        ...
+        
+        my $result = $self->SUPER::render($context, $global_safe);   # in order to render current block contents, invoke the parent renderer
+        
+        ...
+        
+        return $result;     # this is just a string generated by tag. If your tag is silent, just return an empty string
+    }
+
+After creating your tag class, you need to register it in C<DTL::Fast>, so library knew which module to load on tag keyword:
+
+    use DTL::Fast qw(register_tag);
+
+    register_tag( 'my_block_tag' => 'CustomTags::MyBlockTag' );
+    
+Basically, that's all. For more information you may view through the sources of buit-in block tags.
 
 =head1 CUSTOM FILTERS
 
-To do...
+Every filter in the library implemented as a class, inherited from L<C<DTL::Fast::Filter>>:
+
+    package CustomFilters::MyFilter;                    # class name
+    use strict; use utf8; use warnings FATAL => 'all';  # make your code clean
+    use parent 'DTL::Fast::Filter';                     # parent class
+    
+    $DTL::Fast::FILTER_HANDLERS{'myfilter'} = __PACKAGE__;   # register your module as a filter handler
+    
+    # optional parameter parser, invoked on parsing phase.
+    # If your filter is parametrised, you may need to override this method to parse parameters, stored in
+    # the $self->{'parameter'} which is an array of DTL::Fast::Variable objects, one for each parameter
+    sub parse_parameters
+    {
+        my( $self ) = @_;
+        
+        ... here you may process your input parameters...
+        
+        return $self;   # important to return $self
+    }
+    
+    # main method 
+    sub filter
+    {
+        my( 
+            $self,              # filter object reference
+            $filter_manager,    # filter manager object reference
+            $value,             # filtering value
+            $context            # current context for parametrised filters
+        ) = @_;
+        
+        # here you may modify $value. Don't forget to make a shallow copies on arrays and hashes
+        ....
+        
+        return $value;
+    }
+
+After creating your filter class, you need to register it in C<DTL::Fast>, so library knew which module to load on tag keyword:
+
+    use DTL::Fast qw(register_filter);
+
+    register_filter( 'myfilter' => 'CustomFilters::MyFilter' );
+    
+Basically, that's all. For more information you may view through the sources of buit-in filters.
 
 =head1 CUSTOM OPERATORS
 
 To do...
 
-
-
 =head1 INCOMPATIBILITIES WITH DJANGO TEMPLATES
 
 =over
@@ -722,10 +934,10 @@ Template parsing permormance with software cache wiping on each iteration:
 
     Benchmark: timing 5000 iterations of DTL::Fast  , Dotiac::DTL...
     
-    DTL::Fast  :  5 wallclock secs ( 4.52 usr +  0.16 sys =  4.68 CPU) @ 1068.45/s (n=5000)
-    Dotiac::DTL: 41 wallclock secs (38.87 usr +  2.14 sys = 41.01 CPU) @ 121.93/s (n=5000)
+    DTL::Fast  :  5 wallclock secs ( 4.59 usr +  0.12 sys =  4.71 CPU) @ 1061.36/s (n=5000)
+    Dotiac::DTL: 41 wallclock secs (38.66 usr +  2.22 sys = 40.88 CPU) @ 122.30/s (n=5000)
 
-C<DTL::Fast> parsing templates 8.7 times faster, than L<C<Dotiac::DTL>>.
+C<DTL::Fast> parsing templates 8.67 times faster, than L<C<Dotiac::DTL>>.
 
 To run this test, you need to alter L<C<Dotiac::DTL>> module and change declaration of C<my %cache;> to C<our %cache;>. 
     
@@ -733,10 +945,10 @@ Rendering of pre-compiled template (software cache):
 
     Benchmark: timing 3000 iterations of DTL::Fast  , Dotiac::DTL...
     
-    DTL::Fast  : 45 wallclock secs (44.48 usr +  0.36 sys = 44.84 CPU) @ 66.90/s (n=3000)
-    Dotiac::DTL: 53 wallclock secs (53.02 usr +  0.51 sys = 53.52 CPU) @ 56.05/s (n=3000)
+    DTL::Fast  : 34 wallclock secs (33.86 usr +  0.39 sys = 34.25 CPU) @ 87.59/s (n=3000)
+    Dotiac::DTL: 53 wallclock secs (52.93 usr +  0.62 sys = 53.55 CPU) @ 56.02/s (n=3000)
 
-Tests shows, that C<DTL::Fast> works a 20% faster, than L<C<Dotiac::DTL>> in persistent environment.
+Tests shows, that C<DTL::Fast> works a 56% faster, than L<C<Dotiac::DTL>> in persistent environment.
 
 =head2 CGI
 
@@ -744,10 +956,10 @@ This test rendered test template many times by external script, invoked via C<sy
 
     Benchmark: timing 300 iterations of Dotiac render     , Fast cached render, Fast render
     
-    DTL::Fast  : 41 wallclock secs ( 0.01 usr  0.12 sys + 37.09 cusr  4.03 csys = 41.25 CPU) @  7.27/s (n=300)
-    Dotiac::DTL: 49 wallclock secs ( 0.04 usr  0.09 sys + 44.30 cusr  4.45 csys = 48.88 CPU) @  6.14/s (n=300)
+    DTL::Fast  : 40 wallclock secs ( 0.00 usr  0.12 sys + 35.14 cusr  4.98 csys = 40.24 CPU) @  7.45/s (n=300)
+    Dotiac::DTL: 51 wallclock secs ( 0.00 usr  0.12 sys + 38.29 cusr 12.63 csys = 51.04 CPU) @  5.88/s (n=300)
 
-Tests shows, that C<DTL::Fast> works 18% faster, than L<C<Dotiac::DTL>> in CGI environment.
+Tests shows, that C<DTL::Fast> works 27% faster, than L<C<Dotiac::DTL>> in CGI environment.
 
 =head2 DTL::Fast steps performance
 
