@@ -1,9 +1,10 @@
 package DTL::Fast::Variable;
 use strict; use utf8; use warnings FATAL => 'all'; 
+no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 
 use Scalar::Util qw(looks_like_number);
 use DTL::Fast::FilterManager;
-use DTL::Fast::Utils qw(as_bool html_protect);
+use DTL::Fast::Utils qw(as_bool);
 
 sub new
 {
@@ -65,6 +66,7 @@ sub new
     my $self = bless {
         'variable' => [@variable]
         , 'original' => $variable
+        , 'direct_read' => ( scalar @variable == 1 )
         , 'sign' => $sign
         , 'undef' => $undef
         , 'static' => $static
@@ -89,17 +91,28 @@ sub render
     
     if( not $self->{'undef'} )
     {
-        $value = $self->{'static'} ? 
-            $self->{'variable'}->[0]
-            : $context->get($self->{'variable'});
-
+        $value = $self->{'static'} 
+            ? $self->{'variable'}->[0]
+            : $self->{'direct_read'}
+                ? $context->{'ns'}->[-1]->{$self->{'variable'}->[0]}
+                : $context->get($self->{'variable'});
+                
+        while (ref $value eq 'CODE')
+        {
+            $value = 'lvalue' ~~ attributes::get($value) ?
+                $value->()
+                : $value->($self);
+        }
     }
     $value = $self->{'filter_manager'}->filter($value, $context)
         if $self->{'filter_manager'}->{'filters_number'};
     
-    return ( $global_safe or $self->{'filter_manager'}->{'safe'} ) ?
-        $value
-        : html_protect($value);
+    return (
+        not $global_safe 
+        and not $self->{'filter_manager'}->{'safe'}
+    ) 
+        ? DTL::Fast::html_protect($value)
+        : $value;
 }
 
 our $BOOL_PROCESSORS = {
